"use strict";

exports.__esModule = true;
exports.default = void 0;

var _classnames = _interopRequireDefault(require("classnames"));

var _popper = _interopRequireDefault(require("popper.js"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _componentOrElement = _interopRequireDefault(require("prop-types-extra/lib/componentOrElement"));

var _react = _interopRequireDefault(require("react"));

var _reactDom = _interopRequireDefault(require("react-dom"));

var _getContainer = _interopRequireDefault(require("./utils/getContainer"));

var _ownerDocument = _interopRequireDefault(require("./utils/ownerDocument"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; subClass.__proto__ = superClass; }

var displayName = 'Position';
var propTypes = {
  /**
   * A node, element, or function that returns either. The child will be
   * be positioned next to the `target` specified.
   */
  target: _propTypes.default.oneOfType([_componentOrElement.default, _propTypes.default.func]),

  /**
   * How to position the component relative to the target
   */
  placement: _propTypes.default.oneOf([// These are inlined from Popper.placements for docgen.
  'auto-start', 'auto', 'auto-end', 'top-start', 'top', 'top-end', 'right-start', 'right', 'right-end', 'bottom-end', 'bottom', 'bottom-start', 'left-end', 'left', 'left-start']),

  /**
   * "offsetParent" of the component
   */
  container: _propTypes.default.oneOfType([_componentOrElement.default, _propTypes.default.func]),

  /**
   * Minimum spacing in pixels between container border and component border
   */
  containerPadding: _propTypes.default.number,

  /**
   * Whether the position should be changed on each update
   */
  shouldUpdatePosition: _propTypes.default.bool,

  /**
   * @private
   */
  children: _propTypes.default.oneOfType([_propTypes.default.element.isRequired, _propTypes.default.func.isRequired]).isRequired
};
var defaultProps = {
  placement: 'right',
  containerPadding: 0,
  shouldUpdatePosition: false
  /**
   * The Position component calculates the coordinates for its child, to position
   * it relative to a `target` component or node. Useful for creating callouts
   * and tooltips, the Position component injects a `style` props with `left` and
   * `top` values for positioning your component.
   *
   * It also injects "arrow" `left`, and `top` values for styling callout arrows
   * for giving your components a sense of directionality.
   */

};

var Position =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(Position, _React$Component);

  function Position(props, context) {
    var _this;

    _this = _React$Component.call(this, props, context) || this;

    _this.onUpdate = function (_ref) {
      var _arrowPosition;

      var placement = _ref.placement,
          offsets = _ref.offsets,
          styles = _ref.styles;
      var popper = offsets.popper,
          reference = offsets.reference;
      var arrowPositionDirection;
      var arrowPositionDimension;

      if (placement === 'left' || placement === 'right') {
        arrowPositionDirection = 'top';
        arrowPositionDimension = 'height';
      } else {
        arrowPositionDirection = 'left';
        arrowPositionDimension = 'width';
      }

      var popperPosition = popper[arrowPositionDirection];
      var popperSize = popper[arrowPositionDimension];
      var referencePosition = reference[arrowPositionDirection];
      var referenceSize = reference[arrowPositionDimension];
      var popperPositionMin = referencePosition - popperSize;
      var popperPositionMax = referencePosition + referenceSize;
      var arrowPositionRelReverse = (popperPosition - popperPositionMin) / (popperPositionMax - popperPositionMin);
      var arrowPosition = (1 - arrowPositionRelReverse) * popperSize; // A change in placement might cause the positioned element to rerender, so
      // schedule a recalculation of the position.

      if (placement !== _this.state.placement) {
        _this.popper.scheduleUpdate();
      }

      _this.setState({
        placement: placement,
        style: styles,
        position: {
          left: offsets.popper.left,
          top: offsets.popper.top
        },
        arrowPosition: (_arrowPosition = {}, _arrowPosition[arrowPositionDirection] = arrowPosition, _arrowPosition)
      });
    };

    _this.state = _this.getNullState();
    _this._needsFlush = false;
    _this._lastTarget = null;
    _this.popper = null;
    return _this;
  }

  var _proto = Position.prototype;

  _proto.componentDidMount = function componentDidMount() {
    this.updatePosition(this.getTarget());
  };

  _proto.UNSAFE_componentWillReceiveProps = function UNSAFE_componentWillReceiveProps(nextProps) {
    if (nextProps.placement !== this.props.placement) {
      // Do our best to re-render with the intended next placement.
      this.setState({
        placement: nextProps.placement
      });
    }

    this._needsFlush = true;
  };

  _proto.componentDidUpdate = function componentDidUpdate(prevProps) {
    if (!this._needsFlush) {
      return;
    }

    this._needsFlush = false;
    var target = this.getTarget();

    if (target !== this._lastTarget || this.props.placement !== prevProps.placement || this.props.container !== prevProps.container || this.props.containerPadding !== prevProps.containerPadding || this.props.shouldUpdatePosition !== prevProps.shouldUpdatePosition) {
      this.updatePosition(target);
    } else if (this.popper && this.props.shouldUpdatePosition && this.props.children !== prevProps.children) {
      this.popper.scheduleUpdate();
    }
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    if (this.popper) {
      this.popper.destroy();
    }
  };

  _proto.getNullState = function getNullState() {
    return {
      placement: this.props.placement,
      position: {
        visibility: 'hidden'
      },
      arrowPosition: null
    };
  };

  _proto.getTarget = function getTarget() {
    var target = this.props.target;
    target = typeof target === 'function' ? target() : target;
    return target && _reactDom.default.findDOMNode(target) || null;
  };

  _proto.updatePosition = function updatePosition(target) {
    if (this.popper) {
      this.popper.destroy();
    }

    this._lastTarget = target;

    if (!target) {
      this.setState(this.getNullState());
      return;
    }

    var _this$props = this.props,
        placement = _this$props.placement,
        shouldUpdatePosition = _this$props.shouldUpdatePosition,
        container = _this$props.container,
        containerPadding = _this$props.containerPadding;
    var containerNode = (0, _getContainer.default)(container, (0, _ownerDocument.default)(this).body);
    this.popper = new _popper.default(target, _reactDom.default.findDOMNode(this), {
      placement: placement,
      eventsEnabled: shouldUpdatePosition,
      modifiers: {
        preventOverflow: {
          boundariesElement: containerNode,
          padding: containerPadding
        },
        flip: {
          enabled: placement.indexOf('auto') === 0
        },
        computeStyle: {
          enabled: true
        },
        applyStyle: {
          enabled: false
        }
      },
      onCreate: this.onUpdate,
      onUpdate: this.onUpdate
    });
  };

  _proto.render = function render() {
    var _this$props2 = this.props,
        className = _this$props2.className,
        children = _this$props2.children,
        props = _objectWithoutProperties(_this$props2, ["className", "children"]);

    if (typeof children === 'function') return children(this.state);

    var child = _react.default.Children.only(children);

    delete props.target;
    delete props.placement;
    delete props.container;
    delete props.containerPadding;
    delete props.shouldUpdatePosition;
    return _react.default.cloneElement(child, _extends({}, props, this.state, {
      className: (0, _classnames.default)(child.props.className, className)
    }));
  };

  return Position;
}(_react.default.Component);

Position.displayName = displayName;
Position.propTypes = propTypes;
Position.defaultProps = defaultProps;
var _default = Position;
exports.default = _default;
module.exports = exports["default"];